# -*- coding: utf-8 -*-
"""
Created on Fri Jan 24 15:18:31 2025

@author: Matthias Quintern
"""
from cpdctrl.voltage_measurement_device.base import VoltageMeasurementDevice
from cpdctrl.led_control_device.base import LedControlDevice
from cpdctrl.led_script import LedScript
from cpdctrl.utility.data import DataCollector

import time
import datetime

class Measurement:
    def __init__(self, 
                 vm_dev: VoltageMeasurementDevice, 
                 led_dev: LedControlDevice, 
                 led_script: LedScript,
                 data: DataCollector,

        ):
        self.vm_dev = vm_dev
        self.led_dev = led_dev
        self.led_script = led_script  
        self.data = data

    def measure_finite(self, delta_t: 0.1, flush_after:int|None=None, max_measurements=None, verbose=False):
        # if no "time" in header, set the current local time in ISO 8601 format
        # and without microseconds
        if not "time" in self.data.header:
            self.data.header["time"] = datetime.datetime.now().replace(microsecond=0).isoformat()
        self.vm_dev.reset(True)
        try:
            i = 0
            led_val = self.led_script.start()
            t_iter_start = time.time()
            while max_measurements is None or i < max_measurements:
                tval, vval = self.vm_dev.read_value()
                if i == 0:
                    t0 = tval
                tval -= t0
                self.data.add_data(i, tval, vval, led_val)
                print(f"n = {i:6d}, t = {tval: .2f} s, U = {vval: .5f} V, LED = {led_val:03}%" + " "*10, end='\r')
                if flush_after is not None and (i+1) % flush_after == 0:
                    self.data.flush(verbose=verbose)
                # substract the execution time from the sleep time for a more
                # acurate frequency
                dt_sleep = delta_t - (t_iter_start - time.time())
                if dt_sleep > 0:
                    time.sleep(dt_sleep)
                t_iter_start = time.time()
                led_val = self.led_script.get_state()
                i += 1
        except KeyboardInterrupt:
            pass
        self.data.flush(verbose=verbose)
        print("Measurement stopped" + " "*50)